from collections import namedtuple
from aiida.common.extendeddicts import AttributeDict

Cx1Resources = namedtuple('Cx1Resources',
                          ('nnodes_min', 'nnodes_max', 'ncpus', 'mem_gb_default', 'mem_gb_max', 'walltime_hrs'))

# See:
# http://www.imperial.ac.uk/admin-services/ict/self-service/research-support/rcs/computing/high-throughput-computing/job-sizing/
JOB_CLASSES = AttributeDict({
    'throughput_24': Cx1Resources(1, 1, 8, 10, 96, 24),
    'throughput_72': Cx1Resources(1, 1, 8, 10, 96, 72),
    'general_24': Cx1Resources(1, 16, 32, 10, 62, 24),
    'general_72': Cx1Resources(1, 16, 32, 10, 62, 72),
    'singlenode_24': Cx1Resources(1, 1, 48, 10, 124, 24),
    'multinode_24': Cx1Resources(3, 16, 12, 10, 46, 24),
    'multinode_48': Cx1Resources(3, 16, 12, 10, 46, 48),
    'debug': Cx1Resources(1, 1, 8, 10, 96, 0.5)
})


def get_calulation_options(resources, nnodes=None, queue=None, mem_gb=None):
    # type: (Cx1Resources, int, str) -> dict
    """ return dict to parse to calculation `metadata.options`
    """
    assert isinstance(resources, Cx1Resources)
    if nnodes is None:
        nnodes = resources.nnodes_min
    assert resources.nnodes_min <= nnodes <= resources.nnodes_max
    if mem_gb is None:
        mem_gb = resources.mem_gb_default
    assert 1 <= mem_gb <= resources.mem_gb_max

    options = {
        'resources': {
            'num_machines': int(nnodes),
            'num_mpiprocs_per_machine': resources.ncpus,
        },
        'max_memory_kb': int(mem_gb * 1e6),
        'max_wallclock_seconds': int(resources.walltime_hrs * 3600),
        'withmpi': True,
    }

    if queue is not None:
        options['queue_name'] = queue

    return options



import uuid

from aiida_icl.schedulers.pbspro_cx1 import PbsproCx1Scheduler
from aiida.schedulers.datastructures import JobTemplate
from aiida.common.datastructures import CodeInfo, CodeRunMode


def test_submit_script():
    scheduler = PbsproCx1Scheduler()

    job_tmpl = JobTemplate()
    job_tmpl.uuid = str(uuid.uuid4())
    job_tmpl.shebang = '#!/bin/bash -l'
    job_tmpl.job_resource = scheduler.create_job_resource(num_machines=1, num_mpiprocs_per_machine=32)
    job_tmpl.max_memory_kb = 2e6
    job_tmpl.max_wallclock_seconds = 24.5 * 3600
    job_tmpl.queue_name = 'myqueue'
    code_info = CodeInfo()
    code_info.cmdline_params = ['mpirun', '-np', '23', 'pw.x', '-npool', '1']
    code_info.stdin_name = 'aiida.in'
    job_tmpl.codes_info = [code_info]
    job_tmpl.codes_run_mode = CodeRunMode.SERIAL

    submit_script_text = scheduler.get_submit_script(job_tmpl).splitlines()

    assert '#PBS -l select=1:ncpus=32:mem=2gb' in submit_script_text
    assert '#PBS -l walltime=24:30:00' in submit_script_text
    assert '#PBS -q myqueue' in submit_script_text



from aiida.common import NotExistent
from aiida.orm import Computer
from aiida.orm.utils.builders.computer import ComputerBuilder

_COMPUTER_NAME = 'icl_cx1'


def get_cx1_computer(work_dir=None, key_filename=None):
    """get or create a Computer node, to interface with the ICL Cx1 HPC

    Parameters
    ----------
    work_dir :str
        absolute path to the work directory
    key_filename : str
        ssh key filename, e.g. /Users/user_name/.ssh/id_rsa

    Returns
    -------
    aiida.orm.Computer

    """
    try:
        computer_cx1 = Computer.objects.get(name=_COMPUTER_NAME)
    except NotExistent:
        if work_dir is None:
            raise ValueError('work_dir not specified')
        computer_builder = ComputerBuilder(
            label=_COMPUTER_NAME,
            description='Imperial HPC cx1 computer',
            transport='ssh',
            scheduler='pbspro_cx1',
            hostname='login.cx1.hpc.ic.ac.uk',
            prepend_text='',
            append_text='',
            work_dir=work_dir,
            shebang='#!/bin/bash',
            mpiprocs_per_machine=16,
            mpirun_command='mpiexec')
        computer_cx1 = computer_builder.new()
        computer_cx1.store()
        computer_cx1.configure(
            look_for_keys=True,
            key_filename=key_filename,
            timeout=60,
            allow_agent=True,
            compress=True,
            load_system_host_keys=True,
            safe_interval=5.0)
    return computer_cx1



from aiida_icl.utils.computer_cx1 import get_cx1_computer  # noqa: F401
from aiida_icl.utils.crystal_code import get_crystal_code, get_crystal_props_code  # noqa: F401
from aiida_icl.utils.cx1_resources import Cx1Resources, JOB_CLASSES, get_calulation_options  # noqa: F401



from aiida.common import NotExistent
from aiida.orm import Code
from aiida.orm.utils.builders.code import CodeBuilder
from aiida.plugins.entry_point import get_entry_point

_COMPUTER_NAME = 'icl_cx1'
_CRY_CODE_LABEL = 'pcrystal17_v2'
_PROPS_CODE_LABEL = 'pproperties17_v2'
DEFAULT_PCRYSTAL_PATH = ('/rds/general/user/gmallia/home/CRYSTAL17_cx1/v2/bin/Linux-mpiifort_MPP/'
                         'C17-v2_mod_Xeon___mpi__intel-2018___intel-suite__2016.3/Pcrystal')
DEFAULT_PPROPERTIES_PATH = ('/rds/general/user/gmallia/home/CRYSTAL17_cx1/v2/bin/Linux-mpiifort_MPP/'
                            'C17-v2_mod_Xeon___mpi__intel-2018___intel-suite__2016.3/Pproperties')


def get_crystal_code(computer, exec_path=DEFAULT_PCRYSTAL_PATH, modules=('intel-suite/2016.3', 'mpi/intel-5.1')):
    """create a Code node for the CRYSTAL17 code

    Parameters
    ----------
    computer : aiida.orm.Computer
    exec_path : str
        absolute path on the computer, to the crystal executable
    modules : list[str]
        modules to load before execution

    Returns
    -------
    aiida.orm.Code

    """
    try:
        code_cry17 = Code.objects.get(label=_CRY_CODE_LABEL)
    except NotExistent:
        code_builder = CodeBuilder(
            **{
                'label': _CRY_CODE_LABEL,
                'description': 'The CRYSTAL17 code on CX1',
                'code_type': CodeBuilder.CodeType.ON_COMPUTER,
                'computer': computer,
                'prepend_text': 'module load ' + ' '.join(modules),
                'append_text': '',
                'input_plugin': get_entry_point('aiida.calculations', 'crystal17.main'),
                'remote_abs_path': exec_path
            })
        code_cry17 = code_builder.new()
        code_cry17.store()
    return code_cry17


def get_crystal_props_code(computer,
                           exec_path=DEFAULT_PPROPERTIES_PATH,
                           modules=('intel-suite/2016.3', 'mpi/intel-5.1')):
    """create a Code node for the CRYSTAL17 Properties code

    Parameters
    ----------
    computer : aiida.orm.Computer
    exec_path : str
        absolute path on the computer, to the properties executable
    modules : list[str]
        modules to load before execution

    Returns
    -------
    aiida.orm.Code

    """
    try:
        code_props17 = Code.objects.get(label=_PROPS_CODE_LABEL)
    except NotExistent:
        code_builder = CodeBuilder(
            **{
                'label': _PROPS_CODE_LABEL,
                'description': 'The CRYSTAL17 PProperties code on CX1',
                'code_type': CodeBuilder.CodeType.ON_COMPUTER,
                'computer': computer,
                'prepend_text': 'module load ' + ' '.join(modules),
                'append_text': '',
                'input_plugin': get_entry_point('aiida.calculations', 'crystal17.doss'),
                'remote_abs_path': exec_path
            })
        code_props17 = code_builder.new()
        code_props17.store()
    return code_props17



from aiida_icl.utils import (get_cx1_computer, get_crystal_code, get_crystal_props_code, JOB_CLASSES,
                             get_calulation_options)


def test_get_cx1_computer(new_database):
    computer = get_cx1_computer('/test/workdir/')
    assert computer.get_workdir() == '/test/workdir/'
    new_computer = get_cx1_computer()
    assert computer.pk == new_computer.pk


def test_get_crystal_code(new_database):
    computer = get_cx1_computer('/test/workdir/')
    code = get_crystal_code(computer)
    assert code.get_input_plugin_name() == 'crystal17.main'
    new_code = get_crystal_code(computer)
    assert code.pk == new_code.pk


def test_get_crystal_props_code(new_database):
    computer = get_cx1_computer('/test/workdir/')
    code = get_crystal_props_code(computer)
    assert code.get_input_plugin_name() == 'crystal17.doss'
    new_code = get_crystal_props_code(computer)
    assert code.pk == new_code.pk


def test_cx1_resources(data_regression):
    options = get_calulation_options(JOB_CLASSES.general_24)
    data_regression.check(options)



#!/usr/bin/env python

from setuptools import setup, find_packages
import json

if __name__ == '__main__':
    # Provide static information in setup.json
    # such that it can be discovered automatically
    with open('setup.json', 'r') as info:
        kwargs = json.load(info)
    setup(
        packages=find_packages(),
        long_description=open('README.md').read(),
        long_description_content_type='text/markdown',
        **kwargs)
